@@ -15,7 +15,9 @@
 
 function scloseall (sockets)
 
-  reval("sclose(sockets);exit;",sockets);
-  sclose(sockets);
+  ## scloseall (sockets)
+
+  reval ("sclose (sockets); __exit__;", sockets);
+  sclose (sockets);
 
 endfunction
